'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var classnames = require('classnames');
var noop = require('lodash/noop');
var map = require('lodash/map');
var tsReactTag = require('ts-react-tag');
var List = require('@material-ui/core/List');
var ListItem = require('@material-ui/core/ListItem');
var ListItemSecondaryAction = require('@material-ui/core/ListItemSecondaryAction');
var ListItemText = require('@material-ui/core/ListItemText');
var ListItemAvatar = require('@material-ui/core/ListItemAvatar');
var Checkbox = require('@material-ui/core/Checkbox');
var Avatar = require('@material-ui/core/Avatar');
var TextField = require('@material-ui/core/TextField');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var classnames__default = /*#__PURE__*/_interopDefaultLegacy(classnames);
var noop__default = /*#__PURE__*/_interopDefaultLegacy(noop);
var map__default = /*#__PURE__*/_interopDefaultLegacy(map);
var List__default = /*#__PURE__*/_interopDefaultLegacy(List);
var ListItem__default = /*#__PURE__*/_interopDefaultLegacy(ListItem);
var ListItemSecondaryAction__default = /*#__PURE__*/_interopDefaultLegacy(ListItemSecondaryAction);
var ListItemText__default = /*#__PURE__*/_interopDefaultLegacy(ListItemText);
var ListItemAvatar__default = /*#__PURE__*/_interopDefaultLegacy(ListItemAvatar);
var Checkbox__default = /*#__PURE__*/_interopDefaultLegacy(Checkbox);
var Avatar__default = /*#__PURE__*/_interopDefaultLegacy(Avatar);
var TextField__default = /*#__PURE__*/_interopDefaultLegacy(TextField);

var TagContext = React__default['default'].createContext({
    contentHeight: 0,
    onDeselectTag: noop__default['default'],
    onSearch: noop__default['default'],
    onSuggestionClick: noop__default['default'],
    onToggleSelect: noop__default['default'],
    setStateTags: noop__default['default'],
    setStateTagsFiltered: noop__default['default'],
    showSuggestion: true,
    showSuggestionAvatar: true,
    showSuggestionInput: true,
    showSuggestionValue: true,
    showTags: true,
    suggestionHeight: 0,
    suggestionLabel: '',
    tags: [],
    tagsFiltered: [],
});

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

/* eslint-disable require-jsdoc */
function deselectTag(tags, tagId) {
    return map__default['default'](tags, function (tag) {
        if (tag.id === tagId) {
            tag.isSelected = false;
        }
        return tag;
    });
}
function toggleTag(tags, tagId) {
    return map__default['default'](tags, function (tag) {
        if (tag.id === tagId) {
            tag.isSelected = !tag.isSelected;
        }
        return tag;
    });
}
function filterByLabel(tags, searchString) {
    return tags.filter(function (tag) {
        return tag.label.toString().toLowerCase().indexOf(searchString.toLowerCase()) >= 0;
    });
}

var ContentTag = function () {
    var _a = React.useContext(TagContext), contentHeight = _a.contentHeight, onDeselectTag = _a.onDeselectTag, tags = _a.tags, setStateTags = _a.setStateTags;
    /* istanbul ignore next */
    var handleDelete = function (tag) {
        var id = tag.id;
        setStateTags(deselectTag(tags, id));
        onDeselectTag(tags);
    };
    var tagsToRender = map__default['default'](tags, function (tagProps, index) {
        if (!tagProps.isSelected) {
            return null;
        }
        return (React__default['default'].createElement("div", { className: "tag-item", key: "tag-" + index },
            React__default['default'].createElement(tsReactTag.Tag, __assign({}, tagProps, { onDelete: handleDelete }))));
    });
    var style = {
        height: contentHeight,
    };
    return (React__default['default'].createElement("div", { className: "ts-react-tag-select-content-tag", style: style }, tagsToRender));
};

var SuggestionList = function () {
    var _a = React.useContext(TagContext), tags = _a.tags, tagsFiltered = _a.tagsFiltered, onSuggestionClick = _a.onSuggestionClick, onToggleSelect = _a.onToggleSelect, setStateTags = _a.setStateTags, showSuggestionAvatar = _a.showSuggestionAvatar, showSuggestionInput = _a.showSuggestionInput, showSuggestionValue = _a.showSuggestionValue, suggestionHeight = _a.suggestionHeight;
    /* istanbul ignore next */
    var handleToggle = function (tagId) { return function () {
        setStateTags(toggleTag(tags, tagId));
        onToggleSelect(tags);
    }; };
    var listItems = map__default['default'](tagsFiltered, function (tag, index) {
        /* istanbul ignore next */
        var handleOnClick = function () {
            onSuggestionClick(tag);
        };
        /* istanbul ignore if */
        if (!showSuggestionValue) {
            tag.value = undefined;
        }
        return (React__default['default'].createElement(ListItem__default['default'], { key: "iste-item-" + index, button: true, onClick: handleOnClick },
            showSuggestionAvatar && (React__default['default'].createElement(ListItemAvatar__default['default'], null,
                React__default['default'].createElement(Avatar__default['default'], { alt: "Avatar", src: tag.avatarSrc }))),
            React__default['default'].createElement(ListItemText__default['default'], { id: tag.id, primary: tag.label, secondary: tag.value }),
            React__default['default'].createElement(ListItemSecondaryAction__default['default'], null, tag.isSelectable && (React__default['default'].createElement(Checkbox__default['default'], { color: "primary", edge: "end", disabled: tag.disabled, "data-attribute-tag-id": tag.id, onChange: handleToggle(tag.id), checked: tag.isSelected })))));
    });
    var style = {
        height: showSuggestionInput ? suggestionHeight - 50 : /* istanbul ignore next */ suggestionHeight,
    };
    return (React__default['default'].createElement("div", { className: "ts-react-tag-select-suggestion-list-component", style: style },
        React__default['default'].createElement(List__default['default'], { dense: true }, listItems)));
};

var SuggestionFilter = function () {
    var _a = React.useContext(TagContext), onSearch = _a.onSearch, setStateTagsFiltered = _a.setStateTagsFiltered, showSuggestionInput = _a.showSuggestionInput, suggestionLabel = _a.suggestionLabel, tags = _a.tags;
    /* istanbul ignore next */
    var handleFilterChange = function (e) {
        var labelStub = e.target.value;
        var filteredItems = filterByLabel(tags, labelStub);
        setStateTagsFiltered(filteredItems);
        onSearch(filteredItems);
    };
    var style = {
        display: showSuggestionInput ? 'block' : /* istanbul ignore next */ 'none',
    };
    return (React__default['default'].createElement("div", { className: "ts-react-tag-select-suggestion-filter-component", style: style },
        React__default['default'].createElement("div", { className: "filter-wrapper" },
            React__default['default'].createElement(TextField__default['default'], { label: suggestionLabel, variant: "outlined", fullWidth: true, size: "small", onChange: handleFilterChange }))));
};

var Suggestion = function () {
    var suggestionHeight = React.useContext(TagContext).suggestionHeight;
    var style = {
        height: suggestionHeight,
    };
    return (React__default['default'].createElement("div", { className: "ts-react-tag-select-suggestion-component", style: style },
        React__default['default'].createElement(SuggestionFilter, null),
        React__default['default'].createElement(SuggestionList, null)));
};

var Content = function () {
    var _a = React.useContext(TagContext), showSuggestion = _a.showSuggestion, showTags = _a.showTags;
    var styleSuggestion = {
        display: showSuggestion ? 'block' : /* istanbul ignore next */ 'none',
    };
    var styleTags = {
        display: showTags ? 'block' : /* istanbul ignore next */ 'none',
    };
    return (React__default['default'].createElement("div", { className: "ts-react-tag-select-content-component" },
        React__default['default'].createElement("div", { className: "ts-react-tag-select-tag-wrapper", style: styleTags },
            React__default['default'].createElement(ContentTag, null)),
        React__default['default'].createElement("div", { className: "ts-react-tag-select-suggestion-wrapper", style: styleSuggestion },
            React__default['default'].createElement(Suggestion, null))));
};

/* eslint-disable require-jsdoc */
function generateDemoTags() {
    var demoTags = [];
    demoTags.push({
        color: 'default',
        id: 'demo-tag-1',
        label: 'Demo-Tag-1',
        value: 3,
        valueColor: 'primary',
        variant: 'outlined',
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
        avatarSrc: 'http://www.gravatar.com/avatar/3b3be63a4c2a439b013787725dfce802?d=identicon',
    });
    demoTags.push({
        color: 'primary',
        id: 'demo-tag-2',
        label: 'Demo-Tag-2',
        value: 1,
        valueColor: 'secondary',
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
        avatarSrc: 'http://www.gravatar.com/avatar/3b3be63a4c2a439b013787725dfce802?d=identicon',
    });
    demoTags.push({
        color: 'secondary',
        id: 'demo-tag-3',
        label: 'Demo-Tag-3',
        value: -20,
        valueColor: 'primary',
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
        avatarSrc: 'http://www.gravatar.com/avatar/3b3be63a4c2a439b013787725dfce802?d=identicon',
    });
    demoTags.push({
        color: 'secondary',
        id: 'demo-tag-4',
        label: 'Demo-Tag-4 (disabled)',
        value: 6,
        valueColor: 'default',
        disabled: true,
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
        avatarSrc: 'http://www.gravatar.com/avatar/3b3be63a4c2a439b013787725dfce802?d=identicon',
    });
    demoTags.push({
        id: 'demo-tag-5',
        label: 'Demo-Tag-5',
        showValue: false,
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
        avatarSrc: 'http://www.gravatar.com/avatar/3b3be63a4c2a439b013787725dfce802?d=identicon',
    });
    demoTags.push({
        id: 'demo-tag-6',
        label: 'Demo-Tag-6',
        showValue: false,
        showDelete: false,
        showAvatar: false,
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
    });
    demoTags.push({
        id: 'demo-tag-7',
        label: 'Demo-Tag-7 (not selectable)',
        color: 'primary',
        showValue: false,
        showDelete: false,
        showAvatar: false,
        isSelected: false,
        isSelectable: false,
        hideOnDelete: false,
    });
    demoTags.push({
        id: 'demo-tag-8',
        label: 'Demo-Tag-8',
        color: 'primary',
        showValue: false,
        showDelete: false,
        showAvatar: false,
        isSelected: true,
        isSelectable: true,
        hideOnDelete: false,
    });
    return demoTags;
}

var demoTags = generateDemoTags();
var TagSelect = function (_a) {
    var _b = _a.contentHeight /* istanbul ignore next */, contentHeight = _b === void 0 ? 120 : _b /* istanbul ignore next */, _c = _a.id /* istanbul ignore next */, id = _c === void 0 ? 'TsdevReactTagSelect' : _c /* istanbul ignore next */, _d = _a.onDeselectTag /* istanbul ignore next */, onDeselectTag = _d === void 0 ? noop__default['default'] : _d /* istanbul ignore next */, _e = _a.onSearch /* istanbul ignore next */, onSearch = _e === void 0 ? noop__default['default'] : _e /* istanbul ignore next */, _f = _a.onSuggestionClick /* istanbul ignore next */, onSuggestionClick = _f === void 0 ? noop__default['default'] : _f /* istanbul ignore next */, _g = _a.onToggleSelect /* istanbul ignore next */, onToggleSelect = _g === void 0 ? noop__default['default'] : _g /* istanbul ignore next */, _h = _a.showSuggestion /* istanbul ignore next */, showSuggestion = _h === void 0 ? true : _h /* istanbul ignore next */, _j = _a.showSuggestionAvatar /* istanbul ignore next */, showSuggestionAvatar = _j === void 0 ? true : _j /* istanbul ignore next */, _k = _a.showSuggestionInput /* istanbul ignore next */, showSuggestionInput = _k === void 0 ? true : _k /* istanbul ignore next */, _l = _a.showSuggestionValue /* istanbul ignore next */, showSuggestionValue = _l === void 0 ? true : _l /* istanbul ignore next */, _m = _a.showTags /* istanbul ignore next */, showTags = _m === void 0 ? true : _m /* istanbul ignore next */, _o = _a.subClass /* istanbul ignore next */, subClass = _o === void 0 ? '' : _o /* istanbul ignore next */, _p = _a.suggestionHeight /* istanbul ignore next */, suggestionHeight = _p === void 0 ? 240 : _p /* istanbul ignore next */, _q = _a.suggestionLabel /* istanbul ignore next */, suggestionLabel = _q === void 0 ? 'Filter' : _q /* istanbul ignore next */, _r = _a.tags /* istanbul ignore next */, tags = _r === void 0 ? demoTags : _r /* istanbul ignore next */;
    var _s = React.useState(tags), stateTags = _s[0], setStateTags = _s[1];
    var _t = React.useState(tags), stateTagsFiltered = _t[0], setStateTagsFiltered = _t[1];
    /* istanbul ignore next */
    React.useEffect(function () {
        setStateTags(tags);
        setStateTagsFiltered(tags);
    }, [tags]);
    var className = classnames__default['default']('ts-react-tag-select', subClass);
    return (React__default['default'].createElement(TagContext.Provider, { value: {
            contentHeight: contentHeight,
            onDeselectTag: onDeselectTag,
            onSearch: onSearch,
            onSuggestionClick: onSuggestionClick,
            onToggleSelect: onToggleSelect,
            setStateTags: setStateTags,
            setStateTagsFiltered: setStateTagsFiltered,
            showSuggestion: showSuggestion,
            showSuggestionAvatar: showSuggestionAvatar,
            showSuggestionInput: showSuggestionInput,
            showSuggestionValue: showSuggestionValue,
            showTags: showTags,
            suggestionHeight: suggestionHeight,
            suggestionLabel: suggestionLabel,
            tags: stateTags,
            tagsFiltered: stateTagsFiltered,
        } },
        React__default['default'].createElement("div", { id: id, className: className },
            React__default['default'].createElement(Content, null))));
};

exports.TagSelect = TagSelect;
